# February 29 is ignored everywhere
import numpy as np
import matplotlib.pyplot as plt
from scipy import stats
from vg import helpers as my
import vg
from vg.core import monty
from vg import times
from vg.time_series_analysis import time_series as tsa
from vg.time_series_analysis import seasonal_distributions
from vg.time_series_analysis import distributions
from vg import smoothing
from vg.meteo import meteox2y as mxy
import calendar
#import elcom_plotting as ep
#------------------------------------------------------------------------------#
def main(met_=r'D:\daten\vg\in\kn-1980-2000-mit-wind-direction.met',
            plotting=True, var="theta", typ='vg_in', smoothed=False, trend=0,
            average='trig'):
    # load data:
    if var == 'relhum':
        dtimes, temp_d, wd = load_data(met_, typ, "theta")
        dtimes, e_d, wd = load_data(met_, typ, "e")
        t_d = mxy.vap_p2rel(e_d/24.,temp_d/24.)*24.
    else: dtimes, t_d, wd = load_data(met_, typ, var)#
    # get difference from mean and dates seperating episodes
    diff, xi, dtimes_ohne = episoden(dtimes, t_d, wd, plotting=plotting, var=var,
                            smoothed=smoothed, trend=trend, average=average)
    fit_dauer, days = dauer(xi,plotting=plotting, var=var) # get + analyse length of episodes
    # get and analyse amplitudes
    fit_amps, amps = amplitude(diff, xi,plotting=plotting, var=var)
    if plotting: amp_days(days, amps, var=var) # scatter amps over days
    return fit_dauer, days, fit_amps, amps, diff, xi, dtimes_ohne
#------------------------------------------------------------------------------#
def direct_from_ia(mean_arrival,disturbance_std, T=365*21+6, plotting=True,
                    smoothed = False):
    var = "theta"
    diff = vg.interannual_variability(365*21+6,mean_arrival=mean_arrival,
                        disturbance_std= disturbance_std)
    if smoothed:
        print 'smoothing %i !!' %smoothed
        diff = smoothing.smooth(diff,smoothed)
    xx = np.sign(diff[1:]) * np.sign(diff[:-1])
    xi = np.where(xx==-1)[0] # Nullstellen
    if xi[0] != 0: xi = np.append(np.array([0]), xi) # Anfang bei 0
    if xi[-1] < len(diff):xi = np.append(xi, np.array([len(diff)])) # Ende am Ende
    if plotting:
        plt.figure(figsize=(15,4))
        plt.subplots_adjust(bottom=0.15)
        plt.plot(diff, "b-x")
        plt.scatter(xi+0.5,np.zeros_like(xi),c='r')
        plt.grid()
        plt.ylabel(var_units(var))
        plt.title('Distance from average (%s)'%var)
        plt.xlabel('Days since 1. Jan. 1980')
        plt.axhline(y=0.0, color='g')
    fit_dauer, days = dauer(xi,plotting=plotting, var=var) # get + analyse length of episodes
    # get and analyse amplitudes
    fit_amps, amps = amplitude(diff, xi,plotting=plotting, var=var)
    if plotting: amp_days(days, amps, var=var) # scatter amps over days
    return fit_dauer, days, fit_amps, amps, diff, xi
#------------------------------------------------------------------------------#
def direct_from_vg(times_, data, plotting=True, smoothed=False, var = "theta",
                    trend = 0, average = 'trig', m_t = None):
    diff, xi, dtimes_ohne = episoden(times_, data*24., 24, trend=trend,
                                plotting = plotting, var = var, smoothed = smoothed,
                                average = average, m_t = m_t)
    fit_dauer, days = dauer(xi,plotting=plotting, var=var) # get + analyse length of episodes
    # get and analyse amplitudes
    fit_amps, amps = amplitude(diff, xi,plotting=plotting, var=var)
    if plotting: amp_days(days, amps, var=var) # scatter amps over days
    return fit_dauer, days, fit_amps, amps, diff, xi
#------------------------------------------------------------------------------#
def var_units(var):
    if "theta" in var: unit = '$^{\circ}$C'
    elif "ILWR" in var: unit = 'W/m$^{2}$'
    elif "Qsw" in var: unit = 'sw rad rel'
    elif "e " in var: unit = 'hPa'
    elif "relhum" in var: unit = '-'
    elif "u" in var or "v" in var or "U" in var: unit = 'm/s'
    else: unit = '??'
    return unit
#------------------------------------------------------------------------------#
def load_data(met_, typ, var):
    """ loads variable var from file met_
    typ : {'dy','vg_in','vg_out','elcom','cc'}

    Returns
    -------
    dtimes : np.array of datetime objects
    t_d : daily sums of hourly values (24*daily mean)
    """
    wd = 24
    if typ == 'dy':
        varnames = np.array(['Julian','Qsw','ILWR','theta','e','U','R'])
        ii = np.where(varnames==var)
        mdata = np.loadtxt(met_, skiprows=6, usecols=[0,ii[0][0]],
                    converters={0:times.cwr2unix})
        wd = int(86400 / np.round(np.average(mdata[1:,0]- mdata[:-1,0]),0))
##        print 'wd: ', wd
        t_d, dtimes = my.sumup(mdata[:,1], times_=times.unix2datetime(mdata[:,0])
                                , width=wd)
    elif typ == 'elcom':
        varnames = np.array(['Julian','theta','relhum','Qsw','ILWR'])
        ii = np.where(varnames==var)
        mdata = np.loadtxt(met_, skiprows=7, usecols=[0,ii[0][0]],
                    converters={0:times.cwr2unix})
        wd = int(86400 / np.round(np.average(mdata[1:,0]- mdata[:-1,0]),0))
##        print 'wd: ', wd
        t_d, dtimes = my.sumup(mdata[:,1], times_=times.unix2datetime(mdata[:,0])
                                , width=wd)
    elif typ == 'vg_in':
        date, met = vg.read_met(filepath=met_, delimiter="\t")
        t_d, dtimes = my.sumup(met[var], times_=date, width=wd)
    elif typ == 'vg_out':
        met = my.csv2dict(met_, delimiter='\t')
        dtimes = times.str2datetime(met["time"],"%Y-%m-%dT%H:%M:%S")
        t_d = np.array(met[var], dtype=np.float)*24 # sind schon Mittelwerte
    elif typ == 'cc': # D
        met = my.csv2dict(met_, delimiter='\t')
        dtimes = times.str2datetime(met["time"],"%Y-%m-%dT%H:%M:%S")
        t_d = np.array(met[var], dtype=np.float)*24 # sind schon Mittelwerte
    else: print 'typ??'
    return dtimes, t_d, wd
#------------------------------------------------------------------------------#
def episoden(dtimes, t_d, wd, plotting=True, var="theta", smooth_len=False,
                trend = 0, average = 'trig', rtrn_avg = None, m_t = None,
                smoothed='False'):
    """
    Returns
    -------
    diff : value of variable - average of variable for that doy
    xi : indices where diff is crossing zero (seperating episodes)
    """
    temp_daily = []
    dtimes_ohne = np.empty(0) # dtimes ohne 29. 2.
    i = -1
    i_s = []
    for dt in dtimes:
        i = i+1
        if dt.month==2 and dt.day==29: pass#print dt # Schaltjahre rausschmeissen
        else:
            i_s.append(i)
            temp_daily.append(t_d[i]/float(wd) - (i*trend/float(len(dtimes)))) # durch 24 teilen -> Mittelwerte
            dtimes_ohne = np.append(dtimes_ohne, dt)
    temp_daily = np.array(temp_daily)
    if average == 'trig':
        avg = monty.get_means()
        print avg.shape, temp_daily.shape
        diff = temp_daily - avg[i_s]
    elif average=='avg':
        nmax = int(len(temp_daily)//365)*365 # nur ganze Jahre
        temp_daily = temp_daily[:nmax].reshape(-1,365)
        avg = np.average(temp_daily,axis=0)
        diff = (temp_daily - avg).ravel() # Abweichung vom langj. Mittel fuer genau diesen Tag
    else:
        avg = average[i_s]
        diff = temp_daily - average[i_s]
    if var == "Qsw": diff = diff/avg
    if smoothed:
        print 'smoothing %d !!' % smoothed
        diff = smoothing.smooth(diff, smoothed)
    xx = np.sign(diff[1:]) * np.sign(diff[:-1])
    xi = np.where(xx==-1)[0] # Nullstellen
    if xi[0] != 0: xi = np.append(np.array([0]), xi) # Anfang bei 0
    if xi[-1] < len(t_d):xi = np.append(xi, np.array([len(t_d)])) # Ende am Ende
    if plotting:
        plt.figure(figsize=(15,4))
        plt.subplots_adjust(bottom=0.15)
        plt.plot(diff)
        if m_t is not None:
            plt.plot(m_t)
        plt.scatter(xi+0.5,np.zeros_like(xi),c='r')
        plt.grid()
        plt.ylabel(var_units(var))
        plt.title('Distance from average (%s)'%var)
        plt.xlabel('Days since 1. Jan. 1980')
        plt.axhline(y=0.0)
        plt.figure(figsize=(15,4)) # ##
        plt.subplots_adjust(bottom=0.15)
        plt.plot(temp_daily.T)
        plt.plot(avg, linewidth=2, color='k')
        plt.grid()
        plt.ylabel(var_units(var))
        plt.xlabel('Days since 1. Jan. 1980')
    if rtrn_avg: return diff, xi, dtimes_ohne, avg
    else: return diff, xi, dtimes_ohne
#------------------------------------------------------------------------------#
def dauer(xi, plotting=True, var="theta"):
    days = xi[1:] - xi[:-1] #Tage zwischen Nullstellen = Episodendauer
    fit = stats.expon.fit(days, scale=1./days.mean())
##    print 'average episode duration: %6.4f days' %days.mean()
    if plotting:
        plt.figure()
        hists = plt.hist(days, np.max(days-1), normed=True)
        plt.grid()
        plt.title('Duration of Episode (%s)'%var)
        plt.xlabel('Days')
        exfun = stats.expon.pdf(np.arange(np.max(days)), scale=fit[1], loc=fit[0])
        plt.plot(exfun, 'k', linewidth=2)
        x = np.arange(0,50)
        plt.plot(x, 1/fit[1]*np.exp(-(x-fit[0])/fit[1]),
            label=r'$ \frac{1}{%5.3f} \times  exp(\frac{(x-%3.1f)}{%5.3f})$'
            %(fit[1],fit[0], fit[1]))
        plt.legend()
    return fit, days
#------------------------------------------------------------------------------#
def amplitude(diff, xi, plotting=True, var="theta"):
    amps = np.empty(0)
    amps = np.append(amps,np.average(diff[:xi[0]+1]))
    for i in range(2,len(xi)-1):
        amps = np.append(amps,np.average(diff[xi[i-1]+1:xi[i]+1]))
    amps = np.append(amps,np.average(diff[xi[-1]+1:]))
    classes = np.arange(np.floor(np.nanmin(amps)),np.ceil(np.nanmax(amps)), 0.1)
    if len(classes)>300:
        classes = np.arange(np.floor(np.nanmin(amps)),np.ceil(np.nanmax(amps)), 1.)
    elif len(classes)<30:
        classes = np.arange(np.floor(np.nanmin(amps)),np.ceil(np.nanmax(amps)), .01)
    fit = stats.norm.fit(amps)
    print 'standard deviation of amplitude: %6.4f %s'%(amps[np.isfinite(amps)].std(),var_units(var))
    if plotting:
        plt.figure()
        hists=plt.hist(amps, classes, normed=True)
        nofun = stats.norm.pdf(classes, loc=amps[np.isfinite(amps)].mean(), scale=amps[np.isfinite(amps)].std())
        plt.plot(classes, nofun, 'k', linewidth=2)
        plt.grid()
        plt.title('Distance from average (%s)'%var)
        plt.xlabel(var_units(var))
        plt.text(amps[np.isfinite(amps)].std(), np.max(nofun), r'$\sigma = %5.3f$' %amps[np.isfinite(amps)].std())
    return fit, amps
#------------------------------------------------------------------------------#
def amp_days(days, amps, var="theta"):
    plt.figure()
    plt.scatter(days,np.abs(amps),alpha=0.5)
    plt.xlabel('days')
    plt.ylabel('absolut difference %s' %var_units(var))
    plt.title('%s episodes' %var)
    plt.grid()
#------------------------------------------------------------------------------#
def all_amps(filename_, var="theta", typ='dy', nn=100):
    sigma_all_amps = [] #
    for n in range(nn):
        filename = filename_+'%04d.met'%n
        dtimes, t_d = load_data(filename, typ, var)
        diff, xi, dtimes_ohne = episoden(dtimes, t_d,plotting=False, var=var, smoothed=False)
        fit_amps, amps = amplitude(diff, xi,plotting=False, var=var)
        sigma_all_amps.append(fit_amps[1])
    sigma_all_amps = np.array(sigma_all_amps)
    print np.average(sigma_all_amps)
    return sigma_all_amps
#------------------------------------------------------------------------------#
def all_episoden(filename_, var="theta", typ='dy', nn=100, plotting=False,
                 smooth_len=None, rtrn='stats'):
    """ analyse data from nn files in one folder"""
    diff_all, xi_all = np.empty(0), np.empty(0) #
    avg = monty.get_means()
    for n in range(nn):
        print n,
        filename = filename_+'%04d.met'%n
        dtimes, t_d, wd = load_data(filename, typ, var)
        diff, xi, dtimes_ohne = episoden(dtimes, t_d, wd,plotting=False,
                            var=var, smoothed=smooth_len, average=avg)
        xi = xi + len(diff_all)
        diff_all = np.append(diff_all, diff)
        xi_all = np.append(xi_all, xi)
    print '\n',
    if rtrn == 'stats':
        days = xi_all[1:] - xi_all[:-1]
        amps = np.empty(0)
        amps = np.append(amps,np.average(diff_all[:xi_all[0]+1]))
        for i in range(2,len(xi_all)-1):
            amps = np.append(amps,np.average(diff_all[xi_all[i-1]+1:xi_all[i]+1]))
        amps = np.append(amps,np.average(diff_all[xi_all[-1]+1:]))
        print days.mean(), amps[np.isfinite(amps)].std()
        return days.mean(), amps[np.isfinite(amps)].std()
    else:
        fit_dauer, days = dauer(xi_all,plotting=plotting, var=var) # get + analyse length of episodes
        # get and analyse amplitudes
        fit_amps, amps = amplitude(diff_all, xi_all, plotting=plotting, var=var)
        if plotting: amp_days(days, amps, var=var) # scatter amps over days
        return fit_dauer, days, fit_amps, amps, diff_all, xi_all
#------------------------------------------------------------------------------#
def all_episoden_2(filename_, var="theta", typ='dy', nn=100, plotting=False,
                 smooth_len=None, rtrn='stats'):
    """ analyse data from nn files in one folder"""
    days_mean, amps_std, amps_mean = np.empty(0), np.empty(0), np.empty(0) #
    avg = monty.get_means()
    for n in range(nn):# amps[np.isfinite(amps)].std()
        print n,
        filename = filename_+'%04d.met'%n
        all_ = main(met_=filename, typ=typ, average=avg, var=var,plotting=False)
        days_mean = np.append(days_mean, all_[1].mean())
        amps_std = np.append(amps_std, all_[3][np.isfinite(all_[3])].std())
        amps_mean = np.append(amps_mean, all_[3][np.isfinite(all_[3])].mean())
    return days_mean.mean(), amps_std.mean(), amps_mean.mean()
#------------------------------------------------------------------------------#
def autokorrelation_episoden(days, amps, ks):
    auto_days, auto_amps = np.empty(0), np.empty(0)
    for k in ks:
        auto_days = np.append(auto_days, tsa.auto_corr(days, k=k)) # k is not in days, but in episodes. Can be several days
        auto_amps = np.append(auto_amps, tsa.auto_corr(amps, k=k))
    print auto_days.shape, auto_amps.shape
    plt.figure()
    plt.plot(ks,auto_days)
    plt.plot(ks,auto_amps)
    plt.legend(('days','amps'))
    cross = tsa.auto_cov(np.array([days.T, amps.T]),1)
    return auto_days, auto_amps, cross
#------------------------------------------------------------------------------#
def months(met_=r'D:\daten\vg\in\kn-1980-2000-mit-wind-direction.met',
            var="theta", typ='vg_in', smoothed=False, average='trig'):
    # load data:
    if var == 'relhum':
        dtimes, temp_d, wd = load_data(met_, typ, "theta")
        dtimes, e_d , wd= load_data(met_, typ, "e")
        t_d = mxy.vap_p2rel(e_d/24.,temp_d/24.)*24.
    else: dtimes, t_d, wd = load_data(met_, typ, var)
    # get difference from mean and dates seperating episodes
    diff, xi, dtimes_ohne = episoden(dtimes, t_d, wd,plotting=False, var=var,
                        smoothed=smoothed, average=average)
    xdiff = np.arange(len(diff))
    date_xi = dtimes_ohne[xi[:-1]] # letzten weglassen
    xmonth = times.time_part(date_xi, "%m")
    month_ = times.time_part(dtimes_ohne, "%m")
    fit_dauer, days = dauer(xi,plotting=False, var=var) # get + analyse length of episodes
    # get and analyse amplitudes
    fit_amps, amps = amplitude(diff, xi,plotting=False, var=var)
    mcols = ['k',(0,1,1),'c',(0,1,0.5),(0,1,0),'g', 'y', (1,1,0),(1,.5,0),'r',(0.5,0.1,0.1),(.5,.5,.5),'b']
    plt.figure(10,figsize=(15,4)) # diff figure
    plt.plot(dtimes_ohne, diff,c='k', linestyle=':')
    plt.axhline(y=0)
    plt.grid()
    plt.figure(11)# amp figure
    plt.figure(12)# dauer figure
    plt.figure(13)# scatter
    sigma_amp, mw_dau = np.empty(0), np.empty(0)
    for ii in range(1,13):
        print "month: ", ii
        var_ = var + " month " + str(ii)
        xi_mon = np.where(xmonth == ii)
        i_mon = np.where(month_ == ii)
        plt.figure(10)
        # find gaps:
        delta_i = xdiff[i_mon][1:]-xdiff[i_mon][:-1]
        jj = np.where(delta_i>1)[0]
        plt.plot(dtimes_ohne[i_mon][:jj[0]+1],diff[i_mon][:jj[0]+1],c=mcols[ii])
        for j in range(0,len(jj)-1): plt.plot(dtimes_ohne[i_mon][jj[j]+1:jj[j+1]+1],
                                        diff[i_mon][jj[j]+1:jj[j+1]+1],c=mcols[ii])
        plt.plot(dtimes_ohne[i_mon][jj[-1]+1:],diff[i_mon][jj[-1]+1:],c=mcols[ii])
        # amps:
        classes = np.arange(np.floor(np.min(amps[xi_mon])),np.ceil(np.max(amps[xi_mon])), 0.1)
        plt.figure(11)
        hists=plt.hist(amps[xi_mon], classes, normed=True, color=mcols[ii], edgecolor=mcols[ii], alpha=0.2)
        nofun = stats.norm.pdf(classes, loc=amps[xi_mon].mean(), scale=amps[xi_mon].std())
        plt.plot(classes, nofun, c=mcols[ii], linewidth=2, label=r'$\sigma = %5.3f$' %amps[xi_mon].std())
        sigma_amp = np.append(sigma_amp, amps[xi_mon].std())
        # dauer
        fit = stats.expon.fit(days[xi_mon], scale=1./days[xi_mon].mean())
        plt.figure(12)
        hists = plt.hist(days[xi_mon], np.max(days[xi_mon]-1), normed=True, color=mcols[ii], edgecolor=mcols[ii], alpha=0.2)
##        exfun = stats.expon.pdf(np.arange(np.max(days[xi_mon])), scale=fit[1], loc=fit[0])
##        plt.plot(exfun, c=mcols[ii], linewidth=2)
        x = np.arange(0,50)
        plt.plot(x, 1/fit[1]*np.exp(-(x-fit[0])/fit[1]),c=mcols[ii],
                        label="%8.4f"%fit[1])
        mw_dau = np.append(mw_dau, fit[1])
        plt.figure(13)
        plt.scatter(days[xi_mon],np.abs(amps[xi_mon]),c="None", edgecolor=mcols[ii])
    plt.figure(11)
    plt.grid()
    plt.title('Distance from average (%s)'%var)
    plt.xlabel(var_units(var))
    plt.legend()
    plt.figure(12)
    plt.grid()
    plt.title('Duration of Episode (%s)'%var)
    plt.xlabel('Days')
    plt.legend()
    plt.figure(13)
    plt.xlabel('days')
    plt.ylabel('absolut difference %s' %var_units(var))
    plt.title('%s episodes' %var)
    plt.legend(calendar.month_name[1:13])
    plt.grid()
    plt.figure()
    plt.bar(np.arange(1,13),sigma_amp,color='r', edgecolor='r', alpha=0.2)
    ax2 = plt.gca().twinx()
    plt.bar(np.arange(1,13),mw_dau,color='b', edgecolor='b', alpha=0.2)
    plt.grid()
    plt.title('red: sigma_amp, blue: mw_dau')
    return fit_dauer, days, fit_amps, amps, diff, xi
#------------------------------------------------------------------------------#
def episoden_jds(met_=r'D:\daten\vg\in\kn-1980-2000-mit-wind-direction.met',
            typ='vg_in', smoothed=False, average='trig'):
    var="theta"
    # load data:
    dtimes, t_d, wd = load_data(met_, typ, var)
    # get difference from mean and dates seperating episodes
    diff, xi, dtimes_ohne = episoden(dtimes, t_d, wd,plotting=False, var=var,
                        smoothed=smoothed, average=average)
    xdiff = np.arange(len(diff))
    date_xi = dtimes_ohne[xi[:-1]] # letzten weglassen
    xjd = times.datetime2doy(date_xi)
    jd_ = times.datetime2doy(dtimes_ohne)
    fit_dauer, days = dauer(xi,plotting=False, var=var) # get + analyse length of episodes
    # get and analyse amplitudes
    fit_amps, amps = amplitude(diff, xi,plotting=False, var=var)
    sigma_amp, mw_dau = np.empty(0), np.empty(0)
    #
    for jd in range(1,20):
        n_ = 366-19+jd
        ii = np.append(np.where(xjd<=jd+20)[0] ,np.where(xjd>=n_)[0])
        print jd, ii.shape
        sigma_amp = np.append(sigma_amp, amps[ii].std())
        mw_dau = np.append(mw_dau, days[ii].mean())
    for jd in range(20,347):
        ii = np.where(np.abs(xjd-jd)<10)
##        print jd, ii[0].shape
        sigma_amp = np.append(sigma_amp, amps[ii].std())
        mw_dau = np.append(mw_dau, days[ii].mean())
    for jd in range(347,367):
        n_ = 40-(366-(jd-20))
        ii = np.append(np.where(xjd>=jd-20)[0],np.where(xjd<n_)[0])
        print jd, ii.shape
        sigma_amp = np.append(sigma_amp, amps[ii].std())
        mw_dau = np.append(mw_dau, days[ii].mean())
    plt.figure()# amp figure
    plt.scatter(xjd, np.abs(amps), c=days, linewidths=0, alpha=0.5)
    plt.plot(np.arange(1,367),sigma_amp)
    plt.grid()
    plt.title('Amplituden')
    plt.figure()# dauer figure
    plt.scatter(xjd, days, c=np.abs(amps), linewidths=0, alpha=0.5)
    plt.plot(np.arange(1,367),mw_dau)
    plt.grid()
    plt.title('Dauer')
    return xjd, days, amps, mw_dau, sigma_amp
#------------------------------------------------------------------------------#
def years(met_=r'D:\daten\vg\in\kn-1980-2000-mit-wind-direction.met',
            var="theta", typ='vg_in', smoothed=False):
    # load data:
    if var == 'relhum':
        dtimes, temp_d = load_data(met_, typ, "theta")
        dtimes, e_d = load_data(met_, typ, "e")
        t_d = mxy.vap_p2rel(e_d/24.,temp_d/24.)*24.
    else: dtimes, t_d = load_data(met_, typ, var)
    # get difference from mean and dates seperating episodes
    diff, xi, dtimes_ohne = episoden(dtimes, t_d,plotting=False, var=var, smoothed=smoothed)
    xdiff = np.arange(len(diff))
    date_xi = dtimes_ohne[xi[:-1]] # letzten weglassen
    xyears = times.time_part(date_xi, "%Y")
    years_ = times.time_part(dtimes_ohne, "%Y")
    fit_dauer, days = dauer(xi,plotting=False, var=var) # get + analyse length of episodes
    # get and analyse amplitudes
    fit_amps, amps = amplitude(diff, xi,plotting=False, var=var)
    mcols = ['k',(0,0,.5),'b',(0,1,1),'c',(0,1,0.5),(0,1,0),'g', 'y', (1,1,0),(1,.7,0), #10
            (1,.3,0),'r',(0.5,0.1,0.1),(0.8,0.2,0.2),'m',(1,0,1),(1,0.5,1),(.8,.8,.8), #18
            (.5,.5,.5),(.2,.2,.2),'k']
    plt.figure(20,figsize=(15,4)) # diff figure
    plt.plot(dtimes_ohne, diff,c='k', linestyle=':')
    plt.axhline(y=0)
    plt.grid()
    plt.figure(21)# amp figure
    plt.figure(22)# dauer figure
    plt.figure(23)# scatter
    sigma_amp, mw_dau = np.empty(0), np.empty(0)
    for ii in range(np.min(xyears),np.max(xyears)+1):
        print "Year: ", ii
        xi_mon = np.where(xyears == ii)
        i_mon = np.where(years_ == ii)
        plt.figure(20)
        plt.plot(dtimes_ohne[i_mon],diff[i_mon],c=mcols[ii-np.min(xyears)+1])
        # amps:
        classes = np.arange(np.floor(np.min(amps[xi_mon])),np.ceil(np.max(amps[xi_mon])), 0.1)
        plt.figure(21)
        hists=plt.hist(amps[xi_mon], classes, normed=True, color=mcols[ii-np.min(xyears)+1], edgecolor=mcols[ii-np.min(xyears)+1], alpha=0.2)
        nofun = stats.norm.pdf(classes, loc=amps[xi_mon].mean(), scale=amps[xi_mon].std())
        plt.plot(classes, nofun, c=mcols[ii-np.min(xyears)+1], linewidth=2, label=r'$\sigma = %5.3f$' %amps[xi_mon].std())
        sigma_amp = np.append(sigma_amp, amps[xi_mon].std())
        # dauer
        fit = stats.expon.fit(days[xi_mon], scale=1./days[xi_mon].mean())
        plt.figure(22)
        hists = plt.hist(days[xi_mon], np.max(days[xi_mon]-1), normed=True, color=mcols[ii-np.min(xyears)+1], edgecolor=mcols[ii-np.min(xyears)+1], alpha=0.2)
##        exfun = stats.expon.pdf(np.arange(np.max(days[xi_mon])), scale=fit[1], loc=fit[0])
##        plt.plot(exfun, c=mcols[ii-np.min(xyears)+1], linewidth=2)
        x = np.arange(0,50)
        plt.plot(x, 1/fit[1]*np.exp(-(x-fit[0])/fit[1]),c=mcols[ii-np.min(xyears)+1],
                        label="%8.4f"%fit[1])
        mw_dau = np.append(mw_dau, fit[1])
        plt.figure(23)
        plt.scatter(days[xi_mon],np.abs(amps[xi_mon]),c="None", edgecolor=mcols[ii-np.min(xyears)+1])
    plt.figure(21)
    plt.grid()
    plt.title('Distance from average (%s)'%var)
    plt.xlabel(var_units(var))
    plt.legend()
    plt.figure(22)
    plt.grid()
    plt.title('Duration of Episode (%s)'%var)
    plt.xlabel('Days')
    plt.legend()
    plt.figure(23)
    plt.xlabel('days')
    plt.ylabel('absolut difference %s' %var_units(var))
    plt.title('%s episodes' %var)
    plt.legend([str(y_) for y_ in range(np.min(xyears),np.max(xyears)+1)])
    plt.grid()
    plt.figure()
    plt.bar(np.arange(np.min(xyears),np.max(xyears)+1),sigma_amp,color='r', edgecolor='r', alpha=0.2)
    ax2 = plt.gca().twinx()
    plt.bar(np.arange(np.min(xyears),np.max(xyears)+1),mw_dau,color='b', edgecolor='b', alpha=0.2)
    plt.grid()
    plt.title('red: sigma_amp, blue: mw_dau')
    return fit_dauer, days, fit_amps, amps, diff, xi
#------------------------------------------------------------------------------#
def lake_epi(netcs, plotting=True, var="theta", smoothed=False, trend=0, average='avg'):
    # load data:
    dtimes_, data = ep.profile_line_data(netcs,'Locn_4_8_TEMPERATURE',np.arange(43,51))
    t_unix = times.datetime2unix(dtimes_)
    wd = int(86400./np.average(t_unix[1:101]-t_unix[:100]))
    t_d, dtimes = my.sumup(data, times_=dtimes_ , width=wd)
    print 'time steps per day:', wd
    print t_d.shape
    t_d, dtimes = t_d[:,:-2],dtimes[:-2]
    # get difference from mean and dates seperating episodes
    diff, xi, dtimes_ohne = episoden(dtimes, t_d, wd, plotting=plotting, var=var,
                            smoothed=smoothed, trend=trend, average=average)
    fit_dauer, days = dauer(xi,plotting=plotting, var=var) # get + analyse length of episodes
    # get and analyse amplitudes
    fit_amps, amps = amplitude(diff, xi,plotting=plotting, var=var)
    if plotting: amp_days(days, amps, var=var) # scatter amps over days
    return fit_dauer, days, fit_amps, amps, diff, xi
#------------------------------------------------------------------------------#
def dispersion(xi, ndays=30):
    """Ferdis Test auf Zufaelligkeit, Clustering und Periodizitaet

    Parameters
    ----------
    xi : np.array of ints
        Indizes der Nullstellen von diff (aus episoden)
    ndays : int, optional
        Laenge des betrachteten Intervalls in Tagen, default=30

    Returns
    -------
    mues : np.array of ints
        Anzahl der Nullstellen in jedem Intervall
    disp : float
        dispersion"""
    mues = []
    nmax = np.max(xi)/ndays # number of intervals
    print nmax
    for n in range(nmax):
        ii = np.where((xi >= n*ndays) & (xi < (n+1)*ndays))
        mues.append(ii[0].shape)
    mues = np.array(mues)
    disp = np.var(mues, ddof=1) / np.average(mues) - 1.
    print 'Average: ',np.average(mues), '\nvariance: ',np.var(mues, ddof=1)
    print 'Dispersion:', disp
    if np.abs(disp)<0.05: print 'equidispersion (Poisson process)' # the limits are my own invention!
    elif disp<=-0.05: print 'underdispersion (periodicity)' # no scientific background for this 0.05
    elif disp>=0.05: print 'overdispersion (clustering)'
    return mues, disp
#------------------------------------------------------------------------------#


if __name__ == "__main__":
    direct_from_vg(10, 3)

