import unittest
import numpy as np
import vg.time_series_analysis.distributions as ds

distributions = dict([(name, obj) for name, obj in ds.__dict__.items()
                      if isinstance(obj, ds.MyDist)])
dists_frozen = dict([(name, dist(*dist._feasible_start)) 
                     for name, dist in distributions.items()])
quantiles = np.linspace(0, 1, 11)
quantiles_more = np.linspace(0, 1, 500)


class Test(unittest.TestCase):
    def assert_almost_equal_matrix(self, A, B, *args, **kwds):
        """Calls self.assertAlmostEqual on the sequence of flattened A and B.
        *args and **kwds are passed on to self.assertAlmostEqual."""
        flat_tuple = lambda x: tuple(np.asarray(x).ravel())
        for a, b in zip(flat_tuple(A), flat_tuple(B)):
            self.assertAlmostEqual(a, b, *args, **kwds)

    def test_fit(self):
        """Is fit able to reproduce parameters of a ppf-generated sample?"""
        print
        for dist_name, dist in distributions.items():
            print "\t", dist_name
            known_params = dist._feasible_start
            x = dist.ppf(quantiles_more, *known_params)
            fitted_params = dist.fit(x)
            self.assert_almost_equal_matrix(known_params, fitted_params,
                                            places=1)

    def test_roundtrip(self):
        """Is the ppf really the inverse of the cdf?"""
        print
        for dist_name, dist_frozen in dists_frozen.items():
            print "\t", dist_name
            x = dist_frozen.ppf(quantiles)
            quantiles_back = dist_frozen.cdf(x)
            self.assert_almost_equal_matrix(quantiles, quantiles_back)

    def test_scalar(self):
        """Do pdf, cdf and ppf accept and return scalar values?"""
        print
        for dist_name, dist_frozen in dists_frozen.items():
            print "\t", dist_name
            self.assert_(np.isscalar(dist_frozen.pdf(1)))
            self.assert_(np.isscalar(dist_frozen.cdf(1)))
            self.assert_(np.isscalar(dist_frozen.ppf(.5)))

if __name__ == "__main__":
    unittest.main()
